@@ -1,6 +1,7 @@
 #!/bin/sh
 
 test_description='pack-objects breaks long cross-pack delta chains'
+
 . ./test-lib.sh
 
 # This mirrors a repeated push setup:
@@ -75,11 +76,11 @@ test_expect_success 'create series of packs' '
 
 max_chain() {
 	git index-pack --verify-stat-only "$1" >output &&
-	perl -lne '
-	  BEGIN { $len = 0 }
-	  /chain length = (\d+)/ and $len = $1;
-	  END { print $len }
-	' output
+	awk '
+		BEGIN { len=0 }
+		/chain length = [0-9]+:/{ len=$4 }
+		END { print len }
+	' <output | tr -d ':'
 }
 
 # Note that this whole setup is pretty reliant on the current
